@@ -1,6 +1,4 @@
 import contextlib
-import inspect
-import json
 import os
 from datetime import datetime, timezone
 from enum import Enum
@@ -29,7 +27,18 @@ import yaml
 if TYPE_CHECKING:
     cached_property = property
     from yaml import SafeLoader
+
+    Protocol = object
+    runtime_checkable: Callable[..., Any]
 else:
+    try:
+        from typing import Protocol
+    except ImportError:
+        from typing_extensions import Protocol
+    try:
+        from typing import runtime_checkable
+    except ImportError:
+        from typing_extensions import runtime_checkable
     try:
         from yaml import CSafeLoader as SafeLoader
     except ImportError:  # pragma: no cover
@@ -52,11 +61,11 @@ else:
                 return value
 
 
-from .format import PythonVersion
-from .model.pydantic import dump_resolve_reference_action
-from .parser import DefaultPutDict, LiteralType
-from .parser.base import Parser
-from .types import StrictTypes
+from datamodel_code_generator.format import PythonVersion
+from datamodel_code_generator.model.pydantic import dump_resolve_reference_action
+from datamodel_code_generator.parser import DefaultPutDict, LiteralType
+from datamodel_code_generator.parser.base import Parser
+from datamodel_code_generator.types import StrictTypes
 
 T = TypeVar('T')
 
@@ -94,6 +103,8 @@ def snooper_to_methods(  # type: ignore
     max_variable_length=100,
 ) -> Callable[..., Any]:
     def inner(cls: Type[T]) -> Type[T]:
+        import inspect
+
         methods = inspect.getmembers(cls, predicate=inspect.isfunction)
         for name, method in methods:
             snooper_method = pysnooper.snoop(
@@ -122,6 +133,7 @@ def chdir(path: Optional[Path]) -> Iterator[None]:
     else:
         prev_cwd = Path.cwd()
         try:
+
             os.chdir(path if path.is_dir() else path.parent)
             yield
         finally:
@@ -199,12 +211,13 @@ def generate(
     disable_appending_item_suffix: bool = False,
     strict_types: Optional[Sequence[StrictTypes]] = None,
     empty_enum_field_name: Optional[str] = None,
+    custom_class_name_generator: Optional[Callable[[str], str]] = None,
 ) -> None:
     remote_text_cache: DefaultPutDict[str, str] = DefaultPutDict()
     if isinstance(input_, str):
         input_text: Optional[str] = input_
     elif isinstance(input_, ParseResult):
-        from .http import get_body
+        from datamodel_code_generator.http import get_body
 
         input_text = remote_text_cache.get_or_put(
             input_.geturl(), default_factory=get_body
@@ -270,6 +283,8 @@ def generate(
                     )
             except:
                 raise Error('Invalid file format')
+            import json
+
             from genson import SchemaBuilder
 
             builder = SchemaBuilder()
@@ -307,6 +322,7 @@ def generate(
         disable_appending_item_suffix=disable_appending_item_suffix,
         strict_types=strict_types,
         empty_enum_field_name=empty_enum_field_name,
+        custom_class_name_generator=custom_class_name_generator,
     )
 
     with chdir(output):
