@@ -29,7 +29,9 @@ class Imports(DefaultDict[Optional[str], Set[str]]):
 
     def _set_alias(self, from_: Optional[str], imports: Set[str]) -> List[str]:
         return [
-            f'{i} as {self.alias[from_][i]}' if i in self.alias[from_] else i
+            f'{i} as {self.alias[from_][i]}'
+            if i in self.alias[from_] and i != self.alias[from_][i]
+            else i
             for i in sorted(imports)
         ]
 
@@ -63,6 +65,7 @@ IMPORT_LIST = Import.from_full_path('typing.List')
 IMPORT_UNION = Import.from_full_path('typing.Union')
 IMPORT_OPTIONAL = Import.from_full_path('typing.Optional')
 IMPORT_LITERAL = Import.from_full_path('typing.Literal')
+IMPORT_LITERAL_BACKPORT = Import.from_full_path('typing_extensions.Literal')
 IMPORT_SEQUENCE = Import.from_full_path('typing.Sequence')
 IMPORT_MAPPING = Import.from_full_path('typing.Mapping')
 IMPORT_ABC_SEQUENCE = Import.from_full_path('collections.abc.Sequence')
@@ -73,4 +76,5 @@ IMPORT_DICT = Import.from_full_path('typing.Dict')
 IMPORT_DECIMAL = Import.from_full_path('decimal.Decimal')
 IMPORT_DATE = Import.from_full_path('datetime.date')
 IMPORT_DATETIME = Import.from_full_path('datetime.datetime')
+IMPORT_TIME = Import.from_full_path('datetime.time')
 IMPORT_UUID = Import.from_full_path('uuid.UUID')
