@@ -1,5 +1,3 @@
-import keyword
-import re
 from abc import ABC, abstractmethod
 from collections import defaultdict
 from functools import lru_cache
@@ -18,7 +16,7 @@ from typing import (
 )
 
 from jinja2 import Environment, FileSystemLoader, Template
-from pydantic import BaseModel, root_validator
+from pydantic import BaseModel
 
 from datamodel_code_generator import cached_property
 from datamodel_code_generator.imports import IMPORT_OPTIONAL, Import
@@ -95,23 +93,6 @@ class DataModelFieldBase(_BaseModel):
     def method(self) -> Optional[str]:
         return None
 
-    @root_validator
-    def validate_root(cls, values: Any) -> Dict[str, Any]:
-        name: Optional[str] = values.get('name')
-        if name:
-            if keyword.iskeyword(name):
-                alias = name
-                name += '_'
-            elif name.isidentifier():
-                return values
-            else:  # pragma: no cover
-                alias = name
-                name = re.sub(r'\W', '_', name)
-            if not values.get('alias'):
-                values['alias'] = alias
-            values['name'] = name
-        return values
-
     @property
     def represented_default(self) -> str:
         return repr(self.default)
@@ -124,6 +105,20 @@ def get_template(template_file_path: Path) -> Template:
     return environment.get_template(template_file_path.name)
 
 
+def get_module_path(name: str, file_path: Optional[Path]) -> List[str]:
+    if file_path:
+        return [
+            *file_path.parts[:-1],
+            file_path.stem,
+            *name.split('.')[:-1],
+        ]
+    return name.split('.')[:-1]
+
+
+def get_module_name(name: str, file_path: Optional[Path]) -> str:
+    return '.'.join(get_module_path(name, file_path))
+
+
 class TemplateBase(ABC):
     @property
     @abstractmethod
@@ -256,13 +251,11 @@ class DataModel(TemplateBase, ABC):
 
     @property
     def module_path(self) -> List[str]:
-        if self.file_path:
-            return [
-                *self.file_path.parts[:-1],
-                self.file_path.stem,
-                *self.name.split('.')[:-1],
-            ]
-        return self.name.split('.')[:-1]
+        return get_module_path(self.name, self.file_path)
+
+    @property
+    def module_name(self) -> str:
+        return get_module_name(self.name, self.file_path)
 
     @property
     def all_data_types(self) -> Iterator['DataType']:
