@@ -1,7 +1,6 @@
 import re
 from abc import ABC, abstractmethod
 from collections import OrderedDict, defaultdict
-from functools import lru_cache
 from itertools import groupby
 from pathlib import Path
 from typing import (
@@ -14,7 +13,6 @@ from typing import (
     List,
     Mapping,
     Optional,
-    Pattern,
     Sequence,
     Set,
     Tuple,
@@ -25,19 +23,15 @@ from urllib.parse import ParseResult
 
 from pydantic import BaseModel
 
-from datamodel_code_generator.format import CodeFormatter
-
-from ..format import PythonVersion
-from ..imports import IMPORT_ANNOTATIONS, Import, Imports
-from ..model import pydantic as pydantic_model
-from ..model.base import ALL_MODEL, DataModel, DataModelFieldBase
-from ..model.enum import Enum
-from ..reference import ModelResolver, Reference
-from ..types import DataType, DataTypeManager, StrictTypes
-from . import DefaultPutDict, LiteralType
-
-_UNDER_SCORE_1: Pattern[str] = re.compile(r'(.)([A-Z][a-z]+)')
-_UNDER_SCORE_2: Pattern[str] = re.compile('([a-z0-9])([A-Z])')
+from datamodel_code_generator import Protocol, runtime_checkable
+from datamodel_code_generator.format import CodeFormatter, PythonVersion
+from datamodel_code_generator.imports import IMPORT_ANNOTATIONS, Import, Imports
+from datamodel_code_generator.model import pydantic as pydantic_model
+from datamodel_code_generator.model.base import ALL_MODEL, DataModel, DataModelFieldBase
+from datamodel_code_generator.model.enum import Enum
+from datamodel_code_generator.parser import DefaultPutDict, LiteralType
+from datamodel_code_generator.reference import ModelResolver, Reference
+from datamodel_code_generator.types import DataType, DataTypeManager, StrictTypes
 
 escape_characters = str.maketrans(
     {
@@ -52,12 +46,6 @@ escape_characters = str.maketrans(
 )
 
 
-@lru_cache()
-def camel_to_snake(string: str) -> str:
-    subbed = _UNDER_SCORE_1.sub(r'\1_\2', string)
-    return _UNDER_SCORE_2.sub(r'\1_\2', subbed).lower()
-
-
 def to_hashable(item: Any) -> Any:
     if isinstance(item, list):
         return tuple(to_hashable(i) for i in item)
@@ -70,19 +58,6 @@ def to_hashable(item: Any) -> Any:
     return item
 
 
-def snakify_field(field: DataModelFieldBase, model: DataModel) -> None:
-    if not field.name:
-        return
-    original_name = field.name
-    field.name = camel_to_snake(original_name)
-    if field.name != original_name:
-        field.alias = original_name
-
-
-def set_strip_default_none(field: DataModelFieldBase, _: DataModel) -> None:
-    field.strip_default_none = True
-
-
 def dump_templates(templates: List[DataModel]) -> str:
     return '\n\n\n'.join(str(m) for m in templates)
 
@@ -205,11 +180,23 @@ def relative(current_module: str, reference: str) -> Tuple[str, str]:
     return left, right
 
 
+@runtime_checkable
+class Child(Protocol):
+    @property
+    def parent(self) -> Optional[Any]:
+        raise NotImplementedError
+
+
 def get_most_of_parent(value: Any) -> Optional[Any]:
-    if not hasattr(value, 'parent'):
-        return value
-    parent = getattr(value, 'parent')
-    return get_most_of_parent(parent)
+    if isinstance(value, Child):
+        return get_most_of_parent(value.parent)
+    return value
+
+
+def title_to_class_name(title: str) -> str:
+    classname = re.sub('[^A-Za-z0-9]+', ' ', title)
+    classname = ''.join(x for x in classname.title() if not x.isspace())
+    return classname
 
 
 class Result(BaseModel):
@@ -265,6 +252,9 @@ class Parser(ABC):
         disable_appending_item_suffix: bool = False,
         strict_types: Optional[Sequence[StrictTypes]] = None,
         empty_enum_field_name: Optional[str] = None,
+        custom_class_name_generator: Optional[
+            Callable[[str], str]
+        ] = title_to_class_name,
     ):
         self.data_type_manager: DataTypeManager = data_type_manager_type(
             target_python_version,
@@ -300,6 +290,9 @@ class Parser(ABC):
         self.strict_nullable: bool = strict_nullable
         self.use_generic_container_types: bool = use_generic_container_types
         self.enable_faux_immutability: bool = enable_faux_immutability
+        self.custom_class_name_generator: Optional[
+            Callable[[str], str]
+        ] = custom_class_name_generator
 
         self.remote_text_cache: DefaultPutDict[str, str] = (
             remote_text_cache or DefaultPutDict()
@@ -328,18 +321,12 @@ class Parser(ABC):
             self.extra_template_data[ALL_MODEL]['allow_mutation'] = False
 
         self.model_resolver = ModelResolver(
-            aliases=aliases,
             base_url=source.geturl() if isinstance(source, ParseResult) else None,
             singular_name_suffix='' if disable_appending_item_suffix else None,
+            aliases=aliases,
             empty_field_name=empty_enum_field_name,
+            snake_case_field=snake_case_field,
         )
-        self.field_preprocessors: List[
-            Callable[[DataModelFieldBase, DataModel], None]
-        ] = []
-        if self.snake_case_field:
-            self.field_preprocessors.append(snakify_field)
-        if self.strip_default_none:
-            self.field_preprocessors.append(set_strip_default_none)
         self.class_name: Optional[str] = class_name
 
     @property
@@ -365,7 +352,7 @@ class Parser(ABC):
             )
 
     def _get_text_from_url(self, url: str) -> str:
-        from ..http import get_body
+        from datamodel_code_generator.http import get_body
 
         return self.remote_text_cache.get_or_put(url, default_factory=get_body)
 
@@ -376,12 +363,6 @@ class Parser(ABC):
             *url.path.split('/')[1:],
         ]
 
-    def append_result(self, data_model: DataModel) -> None:
-        for field_preprocessor in self.field_preprocessors:
-            for field in data_model.fields:
-                field_preprocessor(field, data_model)
-        self.results.append(data_model)
-
     @property
     def data_type(self) -> Type[DataType]:
         return self.data_type_manager.data_type
@@ -458,9 +439,6 @@ class Parser(ABC):
                     else:
                         model.reference.name = generated_name
 
-        for module, models in module_models:
-            for model in models:
-                model.reference.module_name = '.'.join(module)
         for module, models in module_models:
 
             init = False
@@ -491,7 +469,7 @@ class Parser(ABC):
                         continue
 
                     from_, import_ = full_path = relative(
-                        model.reference.module_name, data_type.full_name
+                        model.module_name, data_type.full_name
                     )
 
                     alias = scoped_model_resolver.add(full_path, import_).name
