@@ -1,5 +1,4 @@
 import enum as _enum
-import sys
 from collections import defaultdict
 from contextlib import contextmanager
 from pathlib import Path
@@ -33,13 +32,17 @@ from datamodel_code_generator import (
 )
 from datamodel_code_generator.format import PythonVersion
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
+from datamodel_code_generator.model import pydantic as pydantic_model
+from datamodel_code_generator.model.base import get_module_name
 from datamodel_code_generator.model.enum import Enum
 from datamodel_code_generator.parser import DefaultPutDict, LiteralType
-
-from ..model import pydantic as pydantic_model
-from ..parser.base import Parser, escape_characters
-from ..reference import Reference, is_url
-from ..types import DataType, DataTypeManager, StrictTypes, Types
+from datamodel_code_generator.parser.base import (
+    Parser,
+    escape_characters,
+    title_to_class_name,
+)
+from datamodel_code_generator.reference import Reference, is_url
+from datamodel_code_generator.types import DataType, DataTypeManager, StrictTypes, Types
 
 
 def get_model_by_path(schema: Dict[str, Any], keys: List[str]) -> Dict[str, Any]:
@@ -70,8 +73,10 @@ json_schema_data_formats: Dict[str, Dict[str, Types]] = {
         'binary': Types.binary,
         'date': Types.date,
         'date-time': Types.date_time,
+        'time': Types.time,
         'password': Types.password,
         'email': Types.email,
+        'idn-email': Types.email,
         'uuid': Types.uuid,
         'uuid1': Types.uuid1,
         'uuid2': Types.uuid2,
@@ -262,6 +267,7 @@ class JsonSchemaParser(Parser):
         disable_appending_item_suffix: bool = False,
         strict_types: Optional[Sequence[StrictTypes]] = None,
         empty_enum_field_name: Optional[str] = None,
+        custom_class_name_generator: Optional[Callable[[str], str]] = None,
     ):
         super().__init__(
             source=source,
@@ -297,6 +303,7 @@ class JsonSchemaParser(Parser):
             disable_appending_item_suffix=disable_appending_item_suffix,
             strict_types=strict_types,
             empty_enum_field_name=empty_enum_field_name,
+            custom_class_name_generator=custom_class_name_generator,
         )
 
         self.remote_object_cache: DefaultPutDict[str, Dict[str, Any]] = DefaultPutDict()
@@ -307,21 +314,11 @@ class JsonSchemaParser(Parser):
 
     @property
     def root_id(self) -> Optional[str]:
-        return self._root_id
+        return self.model_resolver.root_id
 
     @root_id.setter
     def root_id(self, value: Optional[str]) -> None:
-        self._root_id = value
-        if value and '/' in value:
-            self._root_id_base_path = value.rsplit('/', 1)[0]
-        else:
-            self._root_id_base_path = None
-
-        self.model_resolver.set_root_id_base_path(self.root_id_base_path)
-
-    @property
-    def root_id_base_path(self) -> Optional[str]:
-        return self._root_id_base_path
+        self.model_resolver.set_root_id(value)
 
     def should_parse_enum_as_literal(self, obj: JsonSchemaObject) -> bool:
         return self.enum_field_as_literal == LiteralType.All or (
@@ -431,7 +428,11 @@ class JsonSchemaParser(Parser):
             if all_of_item.ref:  # $ref
                 base_classes.append(self.model_resolver.add_ref(all_of_item.ref))
             else:
-                fields.extend(self.parse_object_fields(all_of_item, path))
+                fields.extend(
+                    self.parse_object_fields(
+                        all_of_item, path, get_module_name(name, None),
+                    )
+                )
         # ignore an undetected object
         if ignore_duplicate_model and not fields and len(base_classes) == 1:
             return self.data_type(reference=base_classes[0])
@@ -447,12 +448,12 @@ class JsonSchemaParser(Parser):
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
         )
-        self.append_result(data_model_type)
+        self.results.append(data_model_type)
 
         return self.data_type(reference=reference)
 
     def parse_object_fields(
-        self, obj: JsonSchemaObject, path: List[str]
+        self, obj: JsonSchemaObject, path: List[str], module_name: Optional[str] = None
     ) -> List[DataModelFieldBase]:
         properties: Dict[str, JsonSchemaObject] = (
             {} if obj.properties is None else obj.properties
@@ -460,35 +461,42 @@ class JsonSchemaParser(Parser):
         requires: Set[str] = {*()} if obj.required is None else {*obj.required}
         fields: List[DataModelFieldBase] = []
 
-        for field_name, field in properties.items():
-            original_field_name: str = field_name
+        exclude_field_names: Set[str] = set()
+        for original_field_name, field in properties.items():
+
             constraints: Optional[Mapping[str, Any]] = None
             field_name, alias = self.model_resolver.get_valid_field_name_and_alias(
-                field_name
+                original_field_name, exclude_field_names
             )
+            modular_name = f'{module_name}.{field_name}' if module_name else field_name
+
+            exclude_field_names.add(field_name)
             if field.ref:
                 field_type = self.get_ref_data_type(field.ref)
             elif field.is_array:
                 field_type = self.parse_array_fields(
-                    field_name, field, [*path, field_name]
+                    modular_name, field, [*path, field_name]
                 ).data_type
                 constraints = field.dict()
             elif field.anyOf:
-                field_type = self.parse_any_of(field_name, field, [*path, field_name])
+                field_type = self.parse_any_of(modular_name, field, [*path, field_name])
             elif field.oneOf:
-                field_type = self.parse_one_of(field_name, field, [*path, field_name])
+                field_type = self.parse_one_of(modular_name, field, [*path, field_name])
             elif field.allOf:
                 field_type = self.parse_all_of(
-                    field_name, field, [*path, field_name], ignore_duplicate_model=True
+                    modular_name,
+                    field,
+                    [*path, field_name],
+                    ignore_duplicate_model=True,
                 )
             elif field.is_object:
                 if field.properties:
                     field_type = self.parse_object(
-                        field_name, field, [*path, field_name]
+                        modular_name, field, [*path, field_name]
                     )
                 elif isinstance(field.additionalProperties, JsonSchemaObject):
                     field_class_name = self.model_resolver.add(
-                        [*path, field_name], field_name, class_name=True
+                        [*path, field_name], modular_name, class_name=True
                     ).name
 
                     # TODO: supports other type
@@ -570,7 +578,9 @@ class JsonSchemaParser(Parser):
                 if self.should_parse_enum_as_literal(field):
                     field_type = self.data_type(literals=field.enum)
                 else:
-                    field_type = self.parse_enum(field_name, field, [*path, field_name])
+                    field_type = self.parse_enum(
+                        modular_name, field, [*path, field_name]
+                    )
             else:
                 field_type = self.get_data_type(field)
                 if self.field_constraints:
@@ -596,6 +606,7 @@ class JsonSchemaParser(Parser):
                     nullable=field.nullable
                     if self.strict_nullable and (field.has_default or required)
                     else None,
+                    strip_default_none=self.strip_default_none,
                 )
             )
         return fields
@@ -623,14 +634,16 @@ class JsonSchemaParser(Parser):
         self.set_additional_properties(class_name, additional_properties or obj)
         data_model_type = self.data_model_type(
             reference=reference,
-            fields=self.parse_object_fields(obj, path),
+            fields=self.parse_object_fields(
+                obj, path, get_module_name(class_name, None)
+            ),
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
         )
-        self.append_result(data_model_type)
+        self.results.append(data_model_type)
         return self.data_type(reference=reference)
 
     def parse_array_fields(
@@ -707,10 +720,11 @@ class JsonSchemaParser(Parser):
             required=required,
             constraints=obj.dict(),
             nullable=nullable,
+            strip_default_none=self.strip_default_none,
         )
 
     def parse_array(
-        self, name: str, obj: JsonSchemaObject, path: List[str]
+        self, name: str, obj: JsonSchemaObject, path: List[str],
     ) -> DataType:
         field = self.parse_array_fields(name, obj, [*path, name])
         reference = self.model_resolver.add(path, name, loaded=True)
@@ -723,7 +737,7 @@ class JsonSchemaParser(Parser):
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
         )
-        self.append_result(data_model_root)
+        self.results.append(data_model_root)
         return self.data_type(reference=reference)
 
     def parse_root_type(
@@ -764,6 +778,7 @@ class JsonSchemaParser(Parser):
                     required=required,
                     constraints=obj.dict() if self.field_constraints else {},
                     nullable=obj.nullable if self.strict_nullable else None,
+                    strip_default_none=self.strip_default_none,
                 )
             ],
             custom_base_class=self.base_class,
@@ -771,7 +786,7 @@ class JsonSchemaParser(Parser):
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
         )
-        self.append_result(data_model_root_type)
+        self.results.append(data_model_root_type)
         return self.data_type(reference=reference)
 
     def parse_enum(
@@ -832,6 +847,7 @@ class JsonSchemaParser(Parser):
                     default=default,
                     data_type=self.data_type_manager.get_data_type(Types.any),
                     required=True,
+                    strip_default_none=self.strip_default_none,
                 )
             )
 
@@ -850,7 +866,7 @@ class JsonSchemaParser(Parser):
                 path=self.current_source_path,
                 description=obj.description if self.use_schema_description else None,
             )
-            self.append_result(enum)
+            self.results.append(enum)
             return self.data_type(reference=reference)
 
         root_reference = self.model_resolver.add(
@@ -875,7 +891,7 @@ class JsonSchemaParser(Parser):
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
         )
-        self.append_result(enum)
+        self.results.append(enum)
         data_model_root_type = self.data_model_root_type(
             reference=root_reference,
             fields=[
@@ -887,6 +903,7 @@ class JsonSchemaParser(Parser):
                     default=obj.default,
                     required=False,
                     nullable=True,
+                    strip_default_none=self.strip_default_none,
                 )
             ],
             custom_base_class=self.base_class,
@@ -894,7 +911,7 @@ class JsonSchemaParser(Parser):
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
         )
-        self.append_result(data_model_root_type)
+        self.results.append(data_model_root_type)
         return self.data_type(reference=root_reference)
 
     def _get_ref_body(self, resolved_ref: str) -> Dict[Any, Any]:
@@ -927,7 +944,7 @@ class JsonSchemaParser(Parser):
                 if obj.ref_type == JSONReference.LOCAL:
                     # Local Reference – $ref: '#/definitions/myElement'
                     self.reserved_refs[tuple(self.model_resolver.current_root)].add(ref)  # type: ignore
-                elif self.model_resolver.is_after_load(obj.ref):
+                elif self.model_resolver.is_after_load(ref):
                     self.reserved_refs[tuple(ref.split('#')[0].split('/'))].add(ref)  # type: ignore
                 else:
                     if is_url(ref):
@@ -990,17 +1007,7 @@ class JsonSchemaParser(Parser):
     def root_id_context(self, root_raw: Dict[str, Any]) -> Generator[None, None, None]:
         root_id: Optional[str] = root_raw.get('$id')
         previous_root_id: Optional[str] = self.root_id
-        if root_id:
-            try:
-                resolved_ref = self.model_resolver.resolve_ref(root_id).split('#')[0]
-                self._get_ref_body(resolved_ref)
-            except Exception as e:
-                print(f'Parse $id failed. $id={root_id}\n {str(e)}', file=sys.stderr)
-                self.root_id = None
-            else:
-                self.root_id = root_id
-        else:
-            self.root_id = None
+        self.root_id = root_id if root_id else None
         yield
         self.root_id = previous_root_id
 
@@ -1041,11 +1048,17 @@ class JsonSchemaParser(Parser):
                 self.raw_obj = load_yaml(source.text)
                 if self.class_name:
                     obj_name = self.class_name
+                elif self.custom_class_name_generator:
+                    obj_name = self.custom_class_name_generator(
+                        self.raw_obj.get('title', 'Model')
+                    )
                 else:
                     # backward compatible
                     obj_name = self.raw_obj.get('title', 'Model')
                     if not self.model_resolver.validate_name(obj_name):
-                        raise InvalidClassNameError(obj_name)
+                        obj_name = title_to_class_name(obj_name)
+                if not self.model_resolver.validate_name(obj_name):
+                    raise InvalidClassNameError(obj_name)
                 self._parse_file(self.raw_obj, obj_name, path_parts)
 
         self._resolve_unparsed_json_pointer()
