@@ -71,7 +71,6 @@ class Reference(_BaseModel):
     loaded: bool = True
     source: Optional[Any] = None
     children: List[Any] = []
-    module_name: str = ''
     _exclude_fields: ClassVar = {'children'}
 
     @validator('original_name')
@@ -111,19 +110,70 @@ def context_variable(
         setter(previous_value)
 
 
-class ModelResolver:
+_UNDER_SCORE_1: Pattern[str] = re.compile(r'(.)([A-Z][a-z]+)')
+_UNDER_SCORE_2: Pattern[str] = re.compile('([a-z0-9])([A-Z])')
+
+
+@lru_cache()
+def camel_to_snake(string: str) -> str:
+    subbed = _UNDER_SCORE_1.sub(r'\1_\2', string)
+    return _UNDER_SCORE_2.sub(r'\1_\2', subbed).lower()
+
+
+class FieldNameResolver:
     def __init__(
         self,
         aliases: Optional[Mapping[str, str]] = None,
+        snake_case_field: bool = False,
+        empty_field_name: Optional[str] = None,
+    ):
+        self.aliases: Mapping[str, str] = {} if aliases is None else {**aliases}
+        self.empty_field_name: str = empty_field_name or '_'
+        self.snake_case_field = snake_case_field
+
+    def get_valid_name(self, name: str, excludes: Optional[Set[str]] = None) -> str:
+        if not name:
+            name = self.empty_field_name
+        if name[0] == '#':
+            name = name[1:]
+        # TODO: when first character is a number
+        name = re.sub(r'\W', '_', name)
+        if name[0].isnumeric():
+            name = f'field_{name}'
+        if self.snake_case_field:
+            name = camel_to_snake(name)
+        count = 1
+        if iskeyword(name):
+            name += '_'
+        new_name = name
+        while not new_name.isidentifier() or (excludes and new_name in excludes):
+            new_name = f'{name}_{count}'
+            count += 1
+        return new_name
+
+    def get_valid_field_name_and_alias(
+        self, field_name: str, excludes: Optional[Set[str]] = None
+    ) -> Tuple[str, Optional[str]]:
+        if field_name in self.aliases:
+            return self.aliases[field_name], field_name
+        valid_name = self.get_valid_name(field_name, excludes=excludes)
+        return valid_name, None if field_name == valid_name else field_name
+
+
+class ModelResolver:
+    def __init__(
+        self,
         exclude_names: Set[str] = None,
         duplicate_name_suffix: Optional[str] = None,
         base_url: Optional[str] = None,
         singular_name_suffix: Optional[str] = None,
+        aliases: Optional[Mapping[str, str]] = None,
+        snake_case_field: bool = False,
         empty_field_name: Optional[str] = None,
     ) -> None:
         self.references: Dict[str, Reference] = {}
-        self.aliases: Mapping[str, str] = {} if aliases is None else {**aliases}
         self._current_root: Sequence[str] = []
+        self._root_id: Optional[str] = None
         self._root_id_base_path: Optional[str] = None
         self.ids: DefaultDict[str, Dict[str, str]] = defaultdict(dict)
         self.after_load_files: Set[str] = set()
@@ -133,7 +183,11 @@ class ModelResolver:
         self.singular_name_suffix: str = singular_name_suffix if isinstance(
             singular_name_suffix, str
         ) else SINGULAR_NAME_SUFFIX
-        self.empty_field_name: str = empty_field_name or '_'
+        self.field_name_resolver = FieldNameResolver(
+            aliases=aliases,
+            snake_case_field=snake_case_field,
+            empty_field_name=empty_field_name,
+        )
 
     @property
     def base_url(self) -> Optional[str]:
@@ -167,12 +221,21 @@ class ModelResolver:
         with context_variable(self.set_current_root, self.current_root, current_root):
             yield
 
+    @property
+    def root_id(self) -> Optional[str]:
+        return self._root_id
+
     @property
     def root_id_base_path(self) -> Optional[str]:
         return self._root_id_base_path
 
-    def set_root_id_base_path(self, root_id_base_path: Optional[str]) -> None:
-        self._root_id_base_path = root_id_base_path
+    def set_root_id(self, root_id: Optional[str]) -> None:
+        if root_id and '/' in root_id:
+            self._root_id_base_path = root_id.rsplit('/', 1)[0]
+        else:
+            self._root_id_base_path = None
+
+        self._root_id = root_id
 
     def add_id(self, id_: str, path: Sequence[str]) -> None:
         self.ids['/'.join(self.current_root)][id_] = self.resolve_ref(path)
@@ -184,7 +247,15 @@ class ModelResolver:
             joined_path = self.join_path(path)
         if ID_PATTERN.match(joined_path):
             ref: str = self.ids['/'.join(self.current_root)][joined_path]
-        elif '#' in joined_path:
+        elif (
+            '#' not in joined_path
+            and self.root_id_base_path
+            and self.current_root != path
+        ):
+            ref = f'{self.root_id_base_path}/{joined_path}#'
+        else:
+            if '#' not in joined_path:
+                joined_path += '#'
             if joined_path[0] == '#':
                 joined_path = f'{"/".join(self.current_root)}{joined_path}'
             if self.is_remote_ref(joined_path):
@@ -192,18 +263,14 @@ class ModelResolver:
             else:
                 delimiter = joined_path.index('#')
                 ref = f"{''.join(joined_path[:delimiter])}#{''.join(joined_path[delimiter + 1:])}"
-        elif self.root_id_base_path and self.current_root != path:
-            ref = f'{self.root_id_base_path}/{joined_path}#'
-        else:
-            joined_path = f'{joined_path}#'
-            if self.is_remote_ref(joined_path):
-                ref = f'{"/".join(self.current_root[:-1])}/{joined_path}'
-            else:
-                ref = joined_path
         if self.base_url:
             from .http import join_url
 
             return join_url(self.base_url, ref)
+        if is_url(ref):
+            file_part, path_part = ref.split('#', 1)
+            if file_part == self.root_id:
+                return f'{"/".join(self.current_root)}#{path_part}'
         return ref
 
     def is_remote_ref(self, resolved_ref: str) -> bool:
@@ -214,7 +281,6 @@ class ModelResolver:
         )
 
     def is_after_load(self, ref: str) -> bool:
-        ref = self.resolve_ref(ref)
         if self.is_external_root_ref(ref):
             return ref[:-1] in self.after_load_files
         elif self.is_external_ref(ref):
@@ -247,12 +313,12 @@ class ModelResolver:
         split_ref = ref.rsplit('/', 1)
         if len(split_ref) == 1:
             original_name = Path(
-                split_ref[0][:-1] if self.is_external_root_ref(ref) else split_ref[0]
+                split_ref[0][:-1] if self.is_external_root_ref(path) else split_ref[0]
             ).stem
         else:
             original_name = (
                 Path(split_ref[1][:-1]).stem
-                if self.is_external_root_ref(ref)
+                if self.is_external_root_ref(path)
                 else split_ref[1]
             )
         name = self.get_class_name(original_name, unique=False)
@@ -314,13 +380,16 @@ class ModelResolver:
         self, field_name: str, unique: bool = True, reserved_name: Optional[str] = None
     ) -> str:
         if '.' in field_name:
-            split_name = [self.get_valid_name(n) for n in field_name.split('.')]
+            split_name = [
+                self.field_name_resolver.get_valid_name(n)
+                for n in field_name.split('.')
+            ]
             prefix, field_name = '.'.join(split_name[:-1]), split_name[-1]
             prefix += '.'
         else:
             prefix = ''
 
-        field_name = self.get_valid_name(field_name)
+        field_name = self.field_name_resolver.get_valid_name(field_name)
         upper_camel_name = snake_to_upper_camel(field_name)
         if unique:
             if reserved_name == upper_camel_name:
@@ -355,35 +424,15 @@ class ModelResolver:
     def validate_name(cls, name: str) -> bool:
         return name.isidentifier() and not iskeyword(name)
 
-    def get_valid_name(
-        self, name: str, camel: bool = False, excludes: Optional[Set[str]] = None
-    ) -> str:
-        if not name:
-            name = self.empty_field_name
-        if name[0] == '#':
-            name = name[1:]
-        # TODO: when first character is a number
-        name = re.sub(r'\W', '_', name)
-        if name[0].isnumeric():
-            name = f'field_{name}'
-        count = 1
-        new_name = name
-        while (
-            not new_name.isidentifier()
-            and iskeyword(new_name)
-            or (excludes and new_name in excludes)
-        ):
-            new_name = f'{name}_{count}'
-            count += 1
-        return new_name
+    def get_valid_name(self, name: str, excludes: Optional[Set[str]] = None) -> str:
+        return self.field_name_resolver.get_valid_name(name, excludes)
 
     def get_valid_field_name_and_alias(
-        self, field_name: str
+        self, field_name: str, excludes: Optional[Set[str]] = None
     ) -> Tuple[str, Optional[str]]:
-        if field_name in self.aliases:
-            return self.aliases[field_name], field_name
-        valid_name = self.get_valid_name(field_name)
-        return valid_name, None if field_name == valid_name else field_name
+        return self.field_name_resolver.get_valid_field_name_and_alias(
+            field_name, excludes
+        )
 
 
 @lru_cache()
