@@ -1,13 +1,10 @@
 from collections import OrderedDict
-from pathlib import Path
-from tempfile import NamedTemporaryFile
 from typing import Dict, List, Tuple
 
 import pytest
 
-from datamodel_code_generator import DataModelField, DataTypeManager
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
-from datamodel_code_generator.model.pydantic import BaseModel
+from datamodel_code_generator.model.pydantic import BaseModel, DataModelField
 from datamodel_code_generator.parser.base import Parser, relative, sort_data_models
 from datamodel_code_generator.reference import Reference, snake_to_upper_camel
 from datamodel_code_generator.types import DataType
@@ -204,7 +201,7 @@ def test_snake_to_upper_camel(word, expected):
 
 class D(DataModel):
     def __init__(self, filename: str, data: str, fields: List[DataModelFieldBase]):
-        super().__init__(fields)
+        super().__init__(fields=fields, reference=Reference(''))
         self._data = data
 
     def render(self) -> str:
