@@ -1,15 +1,13 @@
 import json
 from pathlib import Path
 from typing import Dict, List, Optional
-from unittest.mock import Mock, call
+from unittest.mock import call
 
 import pytest
 import yaml
 
-from datamodel_code_generator import DataModelField, DataTypeManager
 from datamodel_code_generator.imports import IMPORT_OPTIONAL, Import
 from datamodel_code_generator.model import DataModelFieldBase
-from datamodel_code_generator.model.pydantic import BaseModel, CustomRootType
 from datamodel_code_generator.parser.base import dump_templates
 from datamodel_code_generator.parser.jsonschema import (
     JsonSchemaObject,
@@ -339,7 +337,7 @@ def test_parse_nested_array():
         ('integer', 'unix-time', 'int', None, None),
         ('number', 'float', 'float', None, None),
         ('number', 'double', 'float', None, None),
-        ('number', 'time', 'time', None, None),
+        ('number', 'time', 'time', 'datetime', 'time'),
         ('string', None, 'str', None, None),
         ('string', 'byte', 'str', None, None),
         ('string', 'binary', 'bytes', None, None),
