@@ -1036,13 +1036,37 @@ def test_main_invalid_model_name_failed(capsys):
                 str(output_file),
                 '--input-file-type',
                 'jsonschema',
+                '--class-name',
+                'with',
             ]
         )
         captured = capsys.readouterr()
         assert return_code == Exit.ERROR
         assert (
             captured.err
-            == 'title=\'1 xyz\' is invalid class name. You have to set `--class-name` option\n'
+            == 'title=\'with\' is invalid class name. You have to set `--class-name` option\n'
+        )
+
+
+@freeze_time('2019-07-26')
+def test_main_invalid_model_name_converted(capsys):
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(JSON_SCHEMA_DATA_PATH / 'invalid_model_name.json'),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        captured = capsys.readouterr()
+        assert return_code == Exit.ERROR
+        assert (
+            captured.err
+            == 'title=\'1Xyz\' is invalid class name. You have to set `--class-name` option\n'
         )
 
 
@@ -1079,9 +1103,7 @@ def test_main_root_id_jsonschema(mocker):
     root_id_response.text = 'dummy'
     person_response = mocker.Mock()
     person_response.text = (JSON_SCHEMA_DATA_PATH / 'person.json').read_text()
-    httpx_get_mock = mocker.patch(
-        'httpx.get', side_effect=[root_id_response, person_response]
-    )
+    httpx_get_mock = mocker.patch('httpx.get', side_effect=[person_response])
     with TemporaryDirectory() as output_dir:
         output_file: Path = Path(output_dir) / 'output.py'
         return_code: Exit = main(
@@ -1100,24 +1122,23 @@ def test_main_root_id_jsonschema(mocker):
             == (EXPECTED_MAIN_PATH / 'main_root_id' / 'output.py').read_text()
         )
         httpx_get_mock.assert_has_calls(
-            [
-                call('https://example.com/root_id.json'),
-                call('https://example.com/person.json'),
-            ]
+            [call('https://example.com/person.json'),]
         )
     with pytest.raises(SystemExit):
         main()
 
 
 @freeze_time('2019-07-26')
-def test_main_root_id_jsonschema_root_id_failed(mocker):
-    httpx_get_mock = mocker.patch('httpx.get', side_effect=[Exception])
+def test_main_root_id_jsonschema_self_refs(mocker):
+    person_response = mocker.Mock()
+    person_response.text = (JSON_SCHEMA_DATA_PATH / 'person.json').read_text()
+    httpx_get_mock = mocker.patch('httpx.get', side_effect=[person_response])
     with TemporaryDirectory() as output_dir:
         output_file: Path = Path(output_dir) / 'output.py'
         return_code: Exit = main(
             [
                 '--input',
-                str(JSON_SCHEMA_DATA_PATH / 'root_id.json'),
+                str(JSON_SCHEMA_DATA_PATH / 'root_id_self_ref.json'),
                 '--output',
                 str(output_file),
                 '--input-file-type',
@@ -1125,12 +1146,13 @@ def test_main_root_id_jsonschema_root_id_failed(mocker):
             ]
         )
         assert return_code == Exit.OK
-        assert (
-            output_file.read_text()
-            == (EXPECTED_MAIN_PATH / 'main_root_id' / 'output.py').read_text()
+        assert output_file.read_text() == (
+            EXPECTED_MAIN_PATH / 'main_root_id' / 'output.py'
+        ).read_text().replace(
+            'filename:  root_id.json', 'filename:  root_id_self_ref.json'
         )
         httpx_get_mock.assert_has_calls(
-            [call('https://example.com/root_id.json'),]
+            [call('https://example.com/person.json'),]
         )
     with pytest.raises(SystemExit):
         main()
@@ -2135,6 +2157,27 @@ def test_main_generate_from_directory():
             assert result == path.read_text()
 
 
+@freeze_time('2019-07-26')
+def test_main_generate_custom_class_name_generator():
+
+    custom_class_name_generator = lambda title: f'Custom{title}'
+
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        input_ = (JSON_SCHEMA_DATA_PATH / 'person.json').relative_to(Path.cwd())
+        assert not input_.is_absolute()
+        generate(
+            input_=input_,
+            input_file_type=InputFileType.JsonSchema,
+            output=output_file,
+            custom_class_name_generator=custom_class_name_generator,
+        )
+
+        assert output_file.read_text() == (
+            EXPECTED_MAIN_PATH / 'main_jsonschema' / 'output.py'
+        ).read_text().replace('Person', 'CustomPerson')
+
+
 @freeze_time('2019-07-26')
 def test_main_http_jsonschema(mocker):
     external_directory = JSON_SCHEMA_DATA_PATH / 'external_files_in_directory'
@@ -2432,3 +2475,28 @@ def test_main_jsonschema_special_enum_empty_enum_field_name():
         )
     with pytest.raises(SystemExit):
         main()
+
+
+@freeze_time('2019-07-26')
+def test_main_jsonschema_special_field_name():
+    with TemporaryDirectory() as output_dir:
+        output_file: Path = Path(output_dir) / 'output.py'
+        return_code: Exit = main(
+            [
+                '--input',
+                str(JSON_SCHEMA_DATA_PATH / 'special_field_name.json'),
+                '--output',
+                str(output_file),
+                '--input-file-type',
+                'jsonschema',
+            ]
+        )
+        assert return_code == Exit.OK
+        assert (
+            output_file.read_text()
+            == (
+                EXPECTED_MAIN_PATH / 'main_jsonschema_special_field_name' / 'output.py'
+            ).read_text()
+        )
+    with pytest.raises(SystemExit):
+        main()
